##
## clirc -- An IRC client in Python.
##
## Copyleft 1998 Teemu Kalvas <chery@s2.org>
##
## This module defines a default policy for lots of events.  There are two
## basic approaches when configuring this, either use an entirely different
## policy module, or add hooks to these functions.
##

## This module will be known as policy at runtime so we cannot actually
## import the modules which are going to be referring back to us, we trust
## the main driver to insert the appropriate module objects in this namespace
## by hand.
## import interface
## import irc

import string
import sys
import random
import regex
import time

import config
import reloader
from event import *

def new_net(net, sock):
    view = interface.new_view('-', (net, ''))
    tag = interface.input_callback(sock, lambda _s, _c, _net=net: \
				   _net.input_ready())
    return (view, {}), tag

## There is no reason all the views should be connected to ircnet objects, but
## at this moment there is nothing else so this function does not test for it.
def user_input(str):
    if str:
	net, chan = interface.current_key()
	if str[0] in config.cmd_chars:
	    f = string.find(str, ' ')
	    if f <> -1:
		cmd = str[1:f]
		tail = str[f + 1:]
	    else:
		cmd = str[1:]
		tail = ''
	    cmd = string.lower(cmd)
	    globals().get('command_' + cmd, DefaultCmd(cmd))(net, chan, tail)
	else:
	    if chan:
		irc.send_msg(net, chan, str)

def display(net, chan, str):
    net.current_view()[1][irc.lower(chan)].insert_text(str)

def display_(net, str):
    net.current_view()[0].insert_text(str)

def _note_ctcp(net, id, target, str):
    f = string.find(str, ' ')
    if f <> -1:
	cmd = str[:f]
	tail = str[f + 1:]
    else:
	cmd = str
	tail = ''
    if cmd == 'ACTION':
	display_action(net, id, target, tail)
    elif cmd == 'PING':
	irc.send_ctcpr(net, id[0], 'PING ' + tail)
    elif cmd == 'VERSION':
	irc.send_ctcpr(net, id[0], 'VERSION ' + config.version)

def _note_ctcpr(net, id, target, str):
    f = string.find(str, ' ')
    if f <> -1:
	cmd = str[:f]
	tail = str[f + 1:]
    else:
	cmd = str
	tail = ''
    if cmd == 'PING':
	display_(net, '*** ping reply from ' + id[0])
    elif cmd == 'VERSION':
	display_(net, '*** version reply from ' + id[0] + ': ' + str)

def _note_join(net, id, chan, mode):
    lchan = irc.lower(chan)
    if id[0] == net.current_nick():
	if not interface.has_view_key((net, chan)):
	    net.current_view()[1][lchan] = interface.new_view(chan, (net,chan))
    else:
	if 'o' not in mode and config.autoop(id[1], lchan):
	    net.mode_enqueue(chan, time.time() + 5.0 + 20.0 * random.random(),
			     0, 'o', id[0])

def _note_leave(net, id, chan, str):
    lchan = irc.lower(chan)
    if id[0] == net.current_nick():
	dict = net.current_view()[1]
	dict[lchan].remove()
	del dict[lchan]
    else:
	if net.mode_queue.has_key(lchan):
	    queue = net.mode_queue[lchan]
	    i = 0
	    while i < len(queue):
		if queue[i][1][2] == id[0]:
		    del queue[i]
		else:
		    i = i + 1

def _note_mode(net, id, chan, mode):
    lchan = irc.lower(chan)
    if net.mode_queue.has_key(lchan):
	t = string.split(mode)
	if t[0] == '+o':
	    queue = net.mode_queue[lchan]
	    i = 0
	    while i < len(queue):
		if queue[i][1][2] == t[1]:
		    del queue[i]
		else:
		    i = i + 1

def _command_join(net, chan, tail):
    if tail:
	irc.send_join(net, tail)
    else:
	irc.send_join(net, chan)

def _command_kick(net, chan, tail):
    f = string.find(tail, ' ')
    if f <> -1:
	irc.send_kick(net, chan, tail[:f], tail[f + 1:])
    else:
	irc.send_kick(net, chan, tail)

def _command_me(net, chan, tail):
    irc.send_ctcp(net, chan, 'ACTION ' + tail)
    display_send_action(net, chan, tail)

def _command_mode(net, chan, tail):
    args = string.split(tail)
    params = args[1:]
    for ch in args[0]:
	if ch == '+':
	    sign = 0
	elif ch == '-':
	    sign = 1
	elif sign:
	    if ch in 'bko':
		irc.send_mode(net, chan, 1, ch, params[0])
		del params[0]
	    else:
		irc.send_mode(net, chan, 1, ch)
	else:
	    if ch in 'bklo':
		irc.send_mode(net, chan, 0, ch, params[0])
		del params[0]
	    else:
		irc.send_mode(net, chan, 0, ch)

def _command_msg(net, chan, tail):
    f = string.find(tail, ' ')
    if f <> -1:
	irc.send_privmsg(net, tail[:f], tail[f + 1:])

def _command_test(net, chan, tail):
    if config.autoop(tail, irc.lower(chan)):
	bool = ''
    else:
	bool = 'not '
    display(net, chan, '*** ' + tail + ' is ' + bool + 'auto-opped.')

class DefaultCmd:
    def __init__(sf, cmd):
	sf.cmd = cmd

    def __call__(sf, net, chan, tail):
	if config.verbose:
	    display(net, chan, '*** unknown command: ' + sf.cmd)

def display_note_join_join(str, id):
    if len(str) >= 10 and str[:10] == '*** join: ':
	return (1, str + ', ' + id)
    return (0, '*** join: ' + id)

def _display_note_join(net, (nick, addr), chan, mode):
    if mode:
	mode = ' +' + mode
    net.current_view()[1][irc.lower(chan)].replace_text(display_note_join_join,
					       nick + mode + ' (' + addr + ')')

def display_note_leave_join(str, id):
    if len(str) >= 11 and str[:11] == '*** leave: ':
	return (1, str + ', ' + id)
    return (0, '*** leave: ' + id)

def _display_note_leave(net, id, chan, str):
    net.current_view()[1][irc.lower(chan)].replace_text(display_note_leave_join, id[0] + ' (' + str + ')')

def display_note_mode_join(str, (nick, mode)):
    l = len(nick) + 14
    if len(str) >= l and str[:l] == '*** mode by ' + nick + ': ':
	return (1, str + ', ' + mode)
    return (0, '*** mode by ' + nick + ': ' + mode)

def _display_note_mode(net, id, chan, mode):
    net.current_view()[1][irc.lower(chan)].replace_text(display_note_mode_join,
							(id[0], mode))

def display_note_quit_join(str, (nick, msg)):
    if len(str) >= 10 and str[:10] == '*** quit: ':
	f = string.find(str, '(')
	if str[f + 1:-1] == msg:
	    return (1, str[:f - 1] + ', ' + nick + ' (' + msg + ')')
    return (0, '*** quit: ' + nick + ' (' + msg + ')')

def _display_note_quit(net, id, str):
    net.current_view()[0].replace_text(display_note_quit_join, (id[0], str))

def _display_note_default(net, str):
    if config.verbose:
	net.current_view()[0].insert_text('*** default: ' + str)

def _reload():
    event_init(globals(),
	       ['command_join', 'command_kick', 'command_leave', 'command_me',
		'command_mode', 'command_msg', 'command_names',
		'command_nick', 'command_quit',
		'command_test', 'command_topic',
		'display_action', 'display_send_action', 'note_ctcp',
		'note_ctcpr', 'display_note_invite', 'note_invite',
		'display_note_join', 'note_join', 'display_note_kick',
		'note_kick', 'display_note_leave', 'note_leave',
		'display_note_mode', 'note_mode', 'display_note_msg',
		'note_msg', 'display_note_names', 'note_names',
		'display_note_nick', 'note_nick', 'note_own_nick',
		'display_note_notice', 'note_notice', 'display_note_privmsg',
		'note_privmsg', 'display_note_privnotice', 'note_privnotice',
		'display_note_quit', 'note_quit', 'display_note_topic',
		'note_topic', 'display_note_topic_', 'note_topic_',
		'display_send_msg', 'display_send_privmsg',
		'note_server_name', 'display_note_default', 'note_default',
		'exception'])

    command_join.post(_command_join)
    command_kick.post(_command_kick)
    command_leave.post(lambda net, chan, tail: irc.send_leave(net, chan, tail))
    command_me.post(_command_me)
    command_mode.post(_command_mode)
    command_msg.post(_command_msg)
    command_names.post(lambda net, chan, tail: irc.send_names(net, chan))
    command_nick.post(lambda net, chan, tail: irc.send_nick(net, tail))
    command_quit.post(lambda net, chan, tail: irc.send_quit(net, tail))
    command_test.post(_command_test)
    command_topic.post(lambda net, chan, tail: irc.send_topic(net, chan, tail))
    display_action.post(lambda net, id, chan, str:
			display(net, chan, '* ' + id[0] + ' ' + str))
    display_send_action.post(lambda net, chan, str:
		     display(net, chan, '* ' + net.current_nick() + ' ' + str))
    note_ctcp.post(_note_ctcp)
    note_ctcpr.post(_note_ctcpr)
    display_note_invite.post(lambda net, id, chan:
			 display_(net, '*** invite by ' + id[0] + ': ' + chan))
    note_invite.post(display_note_invite)
    display_note_join.post(_display_note_join)
    note_join.post([_note_join, display_note_join])
    display_note_kick.post(lambda net, id, chan, target, str:
	display(net, chan, '*** kick by ' + id[0] + ': ' +target+' ('+str+')'))
    note_kick.post(display_note_kick)
    display_note_leave.post(_display_note_leave)
    note_leave.post([display_note_leave, _note_leave])
    display_note_mode.post(_display_note_mode)
    note_mode.post([display_note_mode, _note_mode])
    display_note_msg.post(lambda net, id, chan, str:
			  display(net, chan, '<' + id[0] + '> ' + str))
    note_msg.post(display_note_msg)
    display_note_names.post(lambda net, chan, names:
		display(net, chan, '*** names (' + repr(len(names)) + '):' +
			reduce(lambda x, y: x + ' ' + y[0] + y[1], names, '')))
    note_names.post(display_note_names)
    display_note_nick.post(lambda net, id, nick:
		    display_(net, '*** nick change: ' + id[0] + ' to ' + nick))
    note_nick.post(display_note_nick)
    display_note_notice.post(lambda net, id, chan, str:
			     display(net, chan, '-' + id[0] + '- ' + str))
    note_notice.post(display_note_notice)
    display_note_privmsg.post(lambda net, id, str:
			      display_(net, '*' + id[0] + '* ' + str))
    note_privmsg.post(display_note_privmsg)
    display_note_privnotice.post(lambda net, id, str:
				 display_(net, '-' + id[0] + '- ' + str))
    note_privnotice.post(display_note_privnotice)
    display_note_quit.post(_display_note_quit)
    note_quit.post(display_note_quit)
    display_note_topic.post(lambda net, id, chan, str:
		      display(net, chan, '*** topic by ' + id[0] + ': ' + str))
    note_topic.post(display_note_topic)
    display_note_topic_.post(lambda net, chan, str:
			     display(net, chan, '*** topic: ' + str))
    note_topic_.post(display_note_topic_)
    display_send_msg.post(lambda net, chan, str:
			  display(net, chan, '> ' + str))
    irc.send_msg.pre(display_send_msg)
    display_send_privmsg.post(lambda net, nick, str:
			      display_(net, '-> *' + nick + '* ' + str))
    irc.send_privmsg.pre(display_send_privmsg)
    display_note_default.post(_display_note_default)
    note_default.post(display_note_default)

def _start():
    pass

## End. ##
