##
## clirc -- An IRC client in Python.
##
## Copyleft 1998 Teemu Kalvas <chery@s2.org>
##
## IRC server network interface with assorted event handlers.  These will
## probably be heavily hooked in configuration.
##

import regex
import socket
import string
import sys
import time

import interface
import policy
import reloader
from event import *

lower_trans = string.maketrans('ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]',
			       'abcdefghijklmnopqrstuvwxyz{|}')

line_re = regex.compile('\(:\([^ !]*\)\(!\([^ ]*\)\)? +\)?\([^ ]+\)\(.*\)')

def lower(str):
    return string.translate(str, lower_trans)

def enqueue(queue, time, arg):
    i = 0
    while i < len(queue):
	if time < queue[i][0]:
	    queue.insert(i, (time, arg))
	    if i == 0:
		interface.timeout_latest(time)
	    return
	i = i + 1
    queue.append((time, arg))
    if i == 0:
	interface.timeout_latest(time)

def handle_timeout():
    def collapse_modes(modes):
	return reduce(lambda y, x: y + '+-'[x[0]] + x[1], modes, ' ') + ' ' + \
	       reduce(lambda y, x: y + ' ' + x[2], modes, '')

    now = time.time() + 2.0
    next_time = now + 86400.0
    for net in nets:
	for chan, queue in net.mode_queue.items():
	    batch = []
	    while queue and queue[0][0] < now:
		batch.append(queue[0][1])
		del queue[0]
	    if queue:
		if queue[0][1] < next_time:
		    next_time = queue[0][1]
	    else:
		del net.mode_queue[chan]
	    while len(batch) >= 3:
		net.out('MODE ' + chan + collapse_modes(batch[:3]))
		del batch[:3]
	    if batch:
		net.out('MODE ' + chan + collapse_modes(batch))
    while timer_queue and timer_queue[0][0] < now:
	timer_queue[0][1]()
	del timer_queue[0]
    if timer_queue and timer_queue[0][1] < next_time:
	next_time = timer_queue[0][1]
    return next_time

class Net:
    def __init__(sf, servers, nicks, channels):
	sf.servers = servers
	sf.nicks = nicks
	sf.channels = channels
	sf.data = ''
	sf.out_queue = []
	sf.mode_queue = {}
	sf.connect()
	sf.view, sf.tag = policy.new_net(sf, sf.sock)
	reloader.add_class(sf.__class__, 'irc', 'Net')

    def connect(sf):
	sf.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sf.sock.connect(sf.servers[0])
	sf.out('USER ' + username + ' * * :' + realname)
	sf.out('NICK ' + sf.nicks[0])

    ## Do the funky output thing to avoid nasty deadlocks.  Note that we are
    ## really counting on the fact that the maximum irc payload is shorter
    ## than the socket output buffer.  I am not entirely sure whether Berkeley
    ## sockets are insane or I am, but one of us has surely got to be.
    def true_out(sf):
	sf.sock.send(sf.out_queue[0] + '\r\n')
	del sf.out_queue[0]
	if not sf.out_queue:
	    interface.remove_callback(sf.out_tag)

    def out(sf, text):
	sf.sock.send(text + '\r\n')
#	if not sf.out_queue:
#	    sf.out_tag = interface.output_callback(sf.sock, lambda net=sf: \
#						   net.true_out())
#	    #print 'out tag: ' + repr(sf.out_tag)
#	sf.out_queue.append(text)

    def current_nick(sf):
	return sf.nicks[0]

    def current_name(sf):
	return sf.servers[0][0]

    def current_view(sf):
	return sf.view

    def mode_enqueue(sf, chan, time, sign, ch, arg):
	if not sf.mode_queue.has_key(lower(chan)):
	    sf.mode_queue[lower(chan)] = []
	enqueue(sf.mode_queue[lower(chan)], time, (sign, ch, arg))

    def input_ready(sf):
	try:
	    data = sf.sock.recv(4096)
	    if not data:
		interface.remove_callback(sf.tag)
		disconnected(sf)
		return
	    data = sf.data + data
	    while 1:
		f = string.find(data, '\r\n')
		if f <> -1:
		    sf.input_line(data[:f])
		    data = data[f + 2:]
		else:
		    break
	    sf.data = data
	except:
	    policy.exception()

    def input_line(sf, line):
	if line_re.match(line) <> -1:
	    nick, addr, recv, tail = line_re.group(2, 4, 5, 6)
	    if not addr:
		addr = ''
	    args = []
	    i = 0
	    k = len(tail)
	    while i < k:
		while i < k and tail[i] == ' ':
		    i = i + 1
		if i < k:
		    if tail[i] == ':':
			args.append(tail[i + 1:])
			break
		    j = string.find(tail, ' ', i)
		    if j <> -1:
			args.append(tail[i:j])
			i = j
		    else:
			args.append(tail[i:])
			break
	    recv = string.lower(recv)
	    globals().get('recv_' + recv, DefaultRecv(recv))(sf, (nick, addr),
							     args)

def recv_001(net, id, args):
    policy.note_server_name(net, id[0])
    policy.note_own_nick(net, args[0])
    for chan in net.channels:
	net.out('JOIN ' + chan)

def recv_332(net, id, args):
    policy.note_topic_(net, args[1], args[2])

def recv_353(net, id, args):
    names = []
    for name in string.split(args[3]):
	if name[0] in '@+':
	    names.append((name[0], name[1:]))
	else:
	    names.append(('', name))
    policy.note_names(net, args[2], names)

def recv_433(net, id, args):
    net.nicks = net.nicks[1:] + [net.nicks[0]]
    send_nick(net, net.nicks[0])

def recv_invite(net, id, args):
    policy.note_invite(net, id, args[1])

def recv_join(net, id, args):
    chan = args[0]
    f = string.find(chan, '\007')
    if f == -1:
	mode = ''
    else:
	mode = chan[f + 1:]
	chan = chan[:f]
    policy.note_join(net, id, chan, mode)

def recv_kick(net, id, args):
    policy.note_kick(net, id, args[0], args[1], args[2])

def recv_mode(net, id, args):
    chan = args[0]
    params = args[2:]
    sign = 0
    for ch in args[1]:
	if ch == '+':
	    sign = 0
	elif ch == '-':
	    sign = 1
	elif sign:
	    if ch in 'bko':
		policy.note_mode(net, id, chan, '-' + ch + ' ' + params[0])
		del params[0]
	    else:
		policy.note_mode(net, id, chan, '-' + ch)
	else:
	    if ch in 'bklo':
		policy.note_mode(net, id, chan, '+' + ch + ' ' + params[0])
		del params[0]
	    else:
		policy.note_mode(net, id, chan, '+' + ch)

def recv_nick(net, id, args):
    nick = args[0]
    if id[0] == nick:
	if nick in net.nicks:
	    net.nicks.remove(nick)
	net.nicks.insert(0, nick)
	policy.note_own_nick(net, nick)
    policy.note_nick(net, id, nick)

def recv_notice(net, id, args):
    if args[1][0] == '\001':
	policy.note_ctcpr(net, id, args[0], args[1][1:len(args[1]) - 1])
    elif args[0][0] in '#&':
	policy.note_notice(net, id, args[0], args[1])
    else:
	policy.note_privnotice(net, id, args[1])

def recv_part(net, id, args):
    if len(args) == 2:
	policy.note_leave(net, id, args[0], args[1])
    elif len(args) == 1:
	policy.note_leave(net, id, args[0], '')

def recv_ping(net, id, args):
    i = len(args) - 1
    args[i] = ':' + args[i]
    net.out('PONG ' + reduce(lambda x, y: x + ' ' + y, args))

def recv_privmsg(net, id, args):
    if args[1][0] == '\001':
	policy.note_ctcp(net, id, args[0], args[1][1:len(args[1]) - 1])
    elif args[0][0] in '#&':
	policy.note_msg(net, id, args[0], args[1])
    else:
	policy.note_privmsg(net, id, args[1])

def recv_quit(net, id, args):
    policy.note_quit(net, id, args[0])

def recv_topic(net, id, args):
    policy.note_topic(net, id, args[0], args[1])

class DefaultRecv:
    def __init__(sf, recv):
	sf.recv = recv

    def __call__(sf, net, id, tail):
	if sf.recv not in ['002', '003', '004', '251', '252', '253', '254',
			   '255', '375', '372', '376', '366']:
	    policy.note_default(net, repr((sf.recv, id, tail)))

def _reload():
    event_init(globals(),
	       ['disconnected', 'send_ctcp', 'send_ctcpr', 'send_join',
		'send_kick', 'send_leave', 'send_mode', 'send_msg',
		'send_names', 'send_nick', 'send_privmsg', 'send_quit',
		'send_topic'])

    send_ctcp.post(lambda net, target, str:
		   net.out('PRIVMSG ' + target + ' :\001' + str + '\001'))
    send_ctcpr.post(lambda net, target, str:
		    net.out('NOTICE ' + target + ' :\001' + str + '\001'))
    send_join.post(lambda net, chan: net.out('JOIN ' + chan))
    send_kick.post(lambda net, chan, target, str='':
		   net.out('KICK ' + chan + ' ' + target + ' :' + str))
    send_leave.post(lambda net, chan, str='':
		    net.out('PART ' + chan + ' :' + str))
    send_mode.post(lambda net, chan, sign, ch, arg='':
		   net.mode_enqueue(chan, time.time(), sign, ch, arg))
    send_msg.post(lambda net, chan, str:
		  net.out('PRIVMSG ' + chan + ' :' + str))
    send_names.post(lambda net, chan: net.out('NAMES ' + chan))
    send_nick.post(lambda net, nick: net.out('NICK ' + nick))
    send_privmsg.post(lambda net, nick, str:
		      net.out('PRIVMSG ' + nick + ' :' + str))
    send_quit.post(lambda net, str='': net.out('QUIT :' + str))
    send_topic.post(lambda net, chan, str:
		    net.out('TOPIC ' + chan + ' :' + str))
    interface.set_timeout_handler(handle_timeout)

def _start(init):
    global timer_queue, username, realname, nets

    timer_queue = []
    username = init[0]
    realname = init[1]
    nets = map(lambda net: Net(net[0], net[1], net[2]), init[2])

## End. ##
